
from .file import File


def _method(name):
    def impl(*args, **kwargs):
        file = File._file
        if not file:
            raise RuntimeError('No active Dockerfile')
        getattr(file, name)(*args, **kwargs)

    return impl

ARG = _method('ARG')
COPY = _method('COPY')
ENTRYPOINT = _method('ENTRYPOINT')
ENV = _method('ENV')
FROM = _method('FROM')
LABEL = _method('LABEL')
LN = _method('LN')
LINE = LN
NOTE = _method('NOTE')
RUN = _method('RUN')
WORKDIR = _method('WORKDIR')
USER = _method('USER')



#!/usr/bin/env python
from utz import process
from utz.git.repo import git_repo


def ls():
    return process.lines('git', 'submodule', 'foreach', '--quiet', 'echo $name')


def exists(name):
    return name in ls()


def add(url, path=None):
    if exists(path):
        return
    cmd = ['git', 'submodule', 'add', 'url']
    if path:
        cmd.append(path)
    process.run(cmd)


_submodule_map = None


def git_submodules():
    global _submodule_map
    if not _submodule_map:
        _submodule_map = { s.path: s for s in git_repo().submodules }
    return _submodule_map




import re
from re import escape, finditer


def split(s, ch, max=0):
    '''Split `s` on character `ch`, unescaping backslash-escaped instances of `ch` (and unescaping double-backslashes to
    single backslashes)

    If `max` is passed, `max` groups will be extracted, and a final (max+1)th group will be appended to the returned
    list with the remainder of `s` (with no unescaping performed)
    '''
    idx = 0
    strs = []
    n = 0
    nxt = ''
    escaped_ch = escape(ch)
    rgx = r'(?:\\+%s?|%s)' % (escaped_ch, escaped_ch)
    for m in finditer(rgx, s):
        start, end = m.span()
        nxt += s[idx:start]
        idx = start
        span = s[start:end]
        while span.startswith(r'\\'):
            nxt += '\\'
            span = span[2:]
            idx += 2
        if span == '\\' + ch:
            nxt += ch
            idx += 2
        elif span == ch:
            strs.append(nxt)
            nxt = ''
            idx += 1
            n += 1
            if max == n:
                break
        elif span:
            raise RuntimeError(f'Unexpected end to span {s[start:end]}: {span}; {s=}, {idx=}, {m=}')
        assert idx == end
    nxt += s[idx:]
    strs.append(nxt)
    return strs


def join(strs, ch, max=0):
    '''Escape and `ch`-join a list of strings; inverse of `split` above

    In the returned string:
    - backslashes are escaped (turned into double-backslashes)
    - instances of `ch` are escpaed (preceded by a backslash)

    If `max` is passed and `strs`' length is one more than `max`, the final element of `strs` is taken to to have been a
    final, raw group from a corresponding call to split(…, max=…), and is directly appended to the final string
    (skipping the escaping steps above).
    '''
    escaped_ch = escape(ch)
    def esc(s):
        s = re.sub(r'\\', r'\\\\', s)
        s = re.sub(escaped_ch, '\\' + escaped_ch, s)
        return s

    if max and len(strs) > max:
        if len(strs) > max + 1:
            raise ValueError(f'{len(strs)} strs > {max}+1')
        return ch.join(esc(s) for s in strs[:max]) + ch + strs[max]
    else:
        return ch.join(esc(s) for s in strs)



import typing
from dataclasses import fields


def from_dict(cls, d):
    """See https://stackoverflow.com/a/54769644/544236."""
    try:
        fieldtypes = {f.name: f.type for f in fields(cls)}
        kwargs = {}
        for k, v in d.items():
            fieldtype = fieldtypes[k]
            if typing.get_origin(fieldtype) is list:
                [elem_cls] = typing.get_args(fieldtype)
                if type(v) is not list:
                    raise ValueError(f"Non-list class: {v} ({type(v)})")
                kwargs[k] = [
                    from_dict(elem_cls, _v)
                    for _v in v
                ]
            else:
                kwargs[k] = from_dict(fieldtype, v)
        return cls(**kwargs)
    except TypeError as e:
        return d  # Not a dataclass field



from dataclasses import dataclass
from re import fullmatch


@dataclass
class RGB:
    r: float
    g: float
    b: float

    @staticmethod
    def from_css(css):
        if css[0] == '#':
            css = css[1:]

        m = fullmatch(r'(?P<r>[\da-f]{2})(?P<g>[\da-f]{2})(?P<b>[\da-f]{2})', css)
        if m:
            pcs = [ m['r'], m['g'], m['b'] ]
        else:
            m = fullmatch(r'(?P<r>[\da-f])(?P<g>[\da-f])(?P<b>[\da-f])', css)
            assert m
            pcs = [ m['r'] * 2, m['g'] * 2, m['b'] * 2 ]

        r, g, b = [ int(s, 16) for s in pcs ]
        return RGB(r, g, b)

    def __add__(l, r):
        return RGB(l.r + r.r, l.g + r.g, l.b + r.b)

    def __sub__(l, r):
        return RGB(l.r - r.r, l.g - r.g, l.b - r.b)

    def __mul__(self, f):
        return RGB(self.r * f, self.g * f, self.b * f)

    def __truediv__(self, f):
        return RGB(self.r / f, self.g / f, self.b / f)

    @property
    def css(self):
        return '#%s' % ''.join([ '%02x' % int(f) for f in [ self.r, self.g, self.b ]])


def color_interp(l_idx, l_n, r_colors):
    r_n = len(r_colors) - 1
    r_pos = l_idx / l_n * r_n
    r_idx = int(r_pos)
    r_rem = r_pos - r_idx
    r_colors = [ RGB.from_css(r_color) for r_color in r_colors ]
    cur = r_colors[r_idx]
    if r_rem == 0:
        return cur.css
    nxt = r_colors[r_idx + 1]
    color = cur + (nxt - cur) * r_rem
    return color.css


def colors_lengthen(colors, n):
    return [ color_interp(i, n - 1, colors) for i in range(n) ]


def swatches(colors, sep=' ', width=6):
    """Adapted from https://gist.github.com/wmayner/9b099a0e4a5f8e94f0c6ab2f570187a5"""
    from IPython.display import Markdown, display
    display(Markdown(sep.join(
        f'<span style="font-family: monospace">{color} <span style="color: {color}">{chr(9608)*width}</span></span>'
        for color in colors
    )))



from utz.setup import setup

extras_require = {
    'dt': [
        'click',
        'python-dateutil',
        'pytz',
    ],
    'git': [
        'click',
        'GitPython',
        'PyGithub',
    ],
    'pd': [
        'numpy',
        'pandas',
    ],
    'pdf': [
        'reportlab',
        'pyPDF2',
    ],
    'setup': [
        'setuptools',
    ],
    'test': [
        'pytest',
        'pytest-mock',
    ],
}
extras_require['all'] = sum(extras_require.values(), [ 'pyyaml', ])

setup(
    name="utz",
    version="0.4.2",
    install_requires=open('requirements.txt', 'r').read(),
    extras_require=extras_require,
    url="https://github.com/runsascoded/utz",
)
